@@ -209,16 +209,30 @@ grMatrix <- function(m,dense=FALSE,robust=TRUE) {
 #' @param summaryFunction Function on one argument to summarise the output of the function \code{matrixFunction} (e.g., the largest eigenvector).
 #' @param comparisonFunction Function on two inputs to compute some kind of comparison measure for the output of the function \code{summaryFunction} (e.g., vector correlation, or matrix norm).
 #' 
-#' @return A two-column matrix containing per row the global and local summary statistics for each window.
+#' @return A two-column matrix containing per row the global and local summary statistics for each window. Plotting the correlation data of the returned matrix gives a figure analogously to the figure shown here, which was generated with the example code below.
+#' 
+#' \figure{fig.pdf}{options: width=5in}
 #' 
 #' @importFrom Rdpack reprompt
 #' @references Dmitry Prokopenko, Julian Hecker, Edwin Silverman, Marcello Pagano, Markus Noethen, Christian Dina, Christoph Lange and Heide Fier (2016). Utilizing the Jaccard index to reveal population stratification in sequencing data: a simulation study and an application to the 1000 Genomes Project. Bioinformatics, 32(9):1366-1372.
 #' 
 #' @examples
 #' library(locStra)
-#' m <- matrix(sample(0:1,1000,replace=TRUE),ncol=10)
-#' w <- makeWindows(nrow(m),10,10)
-#' print(fullscan(m,w,covMatrix,powerMethod,cor))
+#' library(Matrix)
+#' data(testdata)
+#' cor2 <- function(x,y) ifelse(sum(x)==0 | sum(y)==0, 0, cor(x,y))
+#' windowSize <- 10000
+#' w <- makeWindows(nrow(testdata),windowSize,windowSize)
+#' resCov <- fullscan(testdata,w,covMatrix,powerMethod,cor2)
+#' resJac <- fullscan(testdata,w,jaccardMatrix,powerMethod,cor2)
+#' resSMx <- fullscan(testdata,w,sMatrix,powerMethod,cor2)
+#' resGRM <- fullscan(testdata,w,grMatrix,powerMethod,cor2)
+#' resAll <- cbind(resCov[,1], resJac[,1], resSMx[,1], resGRM[,1])
+#' xlabel <- "SNP position"
+#' ylabel <- "correlation between global and local eigenvectors"
+#' mainlabel <- paste("window size",windowSize)
+#' matplot(w[,1],abs(resAll),type="b",xlab=xlabel,ylab=ylabel,ylim=c(0,1),main=mainlabel)
+#' legend("topright",legend=c("Cov","Jaccard","s-Matrix","GRM"),pch=paste(1:ncol(resAll)))
 #' 
 #' @export
 fullscan <- function(m,windows,matrixFunction,summaryFunction,comparisonFunction) {
@@ -232,6 +246,7 @@ fullscan <- function(m,windows,matrixFunction,summaryFunction,comparisonFunction
 		matrix_local <- matrixFunction(m[windows[i,1]:windows[i,2],])
 		summary_local <- summaryFunction(matrix_local)
 		res[i,] <- c(comparisonFunction(summary_global,summary_local), comparisonFunction(last_summary,summary_local))
+		last_summary <- summary_local
 	}
 	return(res)
 }
